@@ -1,7 +1,7 @@
 /*
  * GridTools
  *
- * Copyright (c) 2014-2019, ETH Zurich
+ * Copyright (c) 2014-2023, ETH Zurich
  * All rights reserved.
  *
  * Please, refer to the LICENSE file in the root directory.
@@ -26,10 +26,10 @@ namespace gridtools {
 #endif
 #endif
 
-#ifdef __NVCC__
-#define GT_CONSTEXPR
+#ifdef __cpp_consteval
+#define GT_CONSTEVAL consteval
 #else
-#define GT_CONSTEXPR constexpr
+#define GT_CONSTEVAL constexpr
 #endif
 
 #define GT_INTERNAL_ERROR                                                                                       \
@@ -38,11 +38,23 @@ namespace gridtools {
 
 #define GT_INTERNAL_ERROR_MSG(x) GT_INTERNAL_ERROR "\nMessage\n\n" x
 
-#if defined(__NVCC__) && defined(__CUDACC_VER_MAJOR__) && \
-    (__CUDACC_VER_MAJOR__ < 9 || __CUDACC_VER_MAJOR__ == 9 && __CUDACC_VER_MINOR__ < 2)
-#define GT_DECLARE_DEFAULT_EMPTY_CTOR(class_name)                          \
-    __forceinline__ __host__ __device__ constexpr class_name() noexcept {} \
-    static_assert(1, "")
+#ifdef __NVCC__
+#define GT_NVCC_DIAG_STR(x) #x
+#ifdef __NVCC_DIAG_PRAGMA_SUPPORT__
+#define GT_NVCC_DIAG_PUSH_SUPPRESS(x) _Pragma("nv_diagnostic push") _Pragma(GT_NVCC_DIAG_STR(nv_diag_suppress x))
+#define GT_NVCC_DIAG_POP_SUPPRESS(x) _Pragma("nv_diagnostic pop")
 #else
-#define GT_DECLARE_DEFAULT_EMPTY_CTOR(class_name) class_name() = default
+#define GT_NVCC_DIAG_PUSH_SUPPRESS(x) _Pragma(GT_NVCC_DIAG_STR(diag_suppress = x))
+#define GT_NVCC_DIAG_POP_SUPPRESS(x) _Pragma(GT_NVCC_DIAG_STR(diag_default = x))
+#endif
+#else
+#define GT_NVCC_DIAG_PUSH_SUPPRESS(x)
+#define GT_NVCC_DIAG_POP_SUPPRESS(x)
+#endif
+
+#if defined(__NVCC__) && (__CUDACC_VER_MAJOR__ == 12 && __CUDACC_VER_MINOR__ >= 1 && __CUDACC_VER_MINOR__ <= 2)
+// enables workaround for CTAD/constexpr issues in CUDA 12.1, 12.2 (https://github.com/GridTools/gridtools/issues/1766)
+#define GT_NVCC_WORKAROUND_1766 1
+#else
+#define GT_NVCC_WORKAROUND_1766 0
 #endif
